#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>

#include <sys/socket.h>
#include <linux/netlink.h>
#include <netinet/in.h>
#include <arpa/inet.h>

int
create_ksock() {
    int k_sock;
    struct sockaddr_nl src_addr, dest_addr;

    k_sock = socket(PF_NETLINK, SOCK_RAW, 31);
    memset(&src_addr, 0, sizeof(src_addr));
    src_addr.nl_family = AF_NETLINK;
    src_addr.nl_pid = getpid(); /* self pid */

    bind(k_sock, (struct sockaddr*)&src_addr, sizeof(src_addr));

    memset(&dest_addr, 0, sizeof(dest_addr));
    memset(&dest_addr, 0, sizeof(dest_addr));
    dest_addr.nl_family = AF_NETLINK;
    dest_addr.nl_pid = 0; /* For Linux Kernel */
    dest_addr.nl_groups = 0; /* unicast */

    return k_sock;
}

void
get_input(int fd) {
    char buf[5];
    size_t bsize;
    memset(buf, 0, 5);
    bsize = recv(fd, buf, 5, 0);
    printf("Received %u bytes: '%s'\n", (unsigned int)bsize, buf);
}

int
create_tsock() {
    int l_sock, t_sock;
    struct sockaddr_in ip4addr;
    struct sockaddr remote_addr;
    socklen_t raddr_len;

    ip4addr.sin_family = AF_INET;
    ip4addr.sin_port = htons(12345);
    inet_pton(AF_INET, "127.0.0.1", &ip4addr.sin_addr);

    l_sock = socket(PF_INET, SOCK_STREAM, 0);
    bind(l_sock, (struct sockaddr*)&ip4addr, sizeof ip4addr);
    listen(l_sock, 5);

    printf("[XX] listening for connection\n");
    t_sock = accept(l_sock, &remote_addr, &raddr_len);
    printf("[XX] client connected, starting select loop\n");
    send(t_sock, "Hi, please send your data now\n", 30, 0);
    get_input(t_sock);
    return t_sock;
}

int
main(void)
{
    fd_set rfds;
    struct timeval tv;
    int retval;
    int k_sock;
    int t_sock;

    int time_wait = 10;

    FD_ZERO(&rfds);
    FD_SET(0, &rfds);

    /* Wait up to five seconds. */

    k_sock = create_ksock();
    t_sock = create_tsock();

    tv.tv_sec = time_wait;
    tv.tv_usec = 0;

    //FD_SET(k_sock, &rfds);
    (void)k_sock;
    FD_SET(t_sock, &rfds);

    retval = select(1, &rfds, NULL, NULL, &tv);
    /* Don't rely on the value of tv now! */

    if (retval == -1)
       perror("select()");
    else if (retval)
       printf("Data is available now.\n");
       /* FD_ISSET(0, &rfds) will be true. */
    else
       printf("No data within %d seconds (retval %d).\n", time_wait, retval);

    get_input(t_sock);
    exit(EXIT_SUCCESS);
}

